from abc import ABC
from .base_0_envtester_class import Base0EnvtesterClass


class Base01EnvtesterMethvarClass(Base0EnvtesterClass, ABC):
    """
    This is a base class for all tests that want to use the 01_envtester_methvar environment.
    """
    pass
