

class FixtureReturn:
    """helper const class for return values"""
    BALDERGLOB_SESSION = "balderglob_session_fixt"
    BALDERGLOB_SETUP = "balderglob_setup_fixt"
    BALDERGLOB_SCENARIO = "balderglob_scenario_fixt"
    BALDERGLOB_VARIATION = "balderglob_variation_fixt"
    BALDERGLOB_TESTCASE = "balderglob_testcase_fixt"

    SETUP_A_SESSION = "setup_a_session_fixt"
    SETUP_A_SETUP = "setup_a_setup_fixt"
    SETUP_A_SCENARIO = "setup_a_scenario_fixt"
    SETUP_A_VARIATION = "setup_a_variation_fixt"
    SETUP_A_TESTCASE = "setup_a_testcase_fixt"

    SETUP_B_SESSION = "setup_b_session_fixt"
    SETUP_B_SETUP = "setup_b_setup_fixt"
    SETUP_B_SCENARIO = "setup_b_scenario_fixt"
    SETUP_B_VARIATION = "setup_b_variation_fixt"
    SETUP_B_TESTCASE = "setup_b_testcase_fixt"

    SCENARIO_A_SESSION = "scenario_a_session_fixt"
    SCENARIO_A_SETUP = "scenario_a_setup_fixt"
    SCENARIO_A_SCENARIO = "scenario_a_scenario_fixt"
    SCENARIO_A_VARIATION = "scenario_a_variation_fixt"
    SCENARIO_A_TESTCASE = "scenario_a_testcase_fixt"

    SCENARIO_B_SESSION = "scenario_b_session_fixt"
    SCENARIO_B_SETUP = "scenario_b_setup_fixt"
    SCENARIO_B_SCENARIO = "scenario_b_scenario_fixt"
    SCENARIO_B_VARIATION = "scenario_b_variation_fixt"
    SCENARIO_B_TESTCASE = "scenario_b_testcase_fixt"
