

class FixtureReturn:
    """helper const class for return values"""
    BALDERGLOB_SESSION = "balderglob_session_fixt"
    BALDERGLOB_SETUP = "balderglob_setup_fixt"
    BALDERGLOB_SCENARIO = "balderglob_scenario_fixt"
    BALDERGLOB_VARIATION = "balderglob_variation_fixt"
    BALDERGLOB_TESTCASE = "balderglob_testcase_fixt"

    SETUP_SESSION = "setup_session_fixt"
    SETUP_SETUP = "setup_setup_fixt"
    SETUP_SCENARIO = "setup_scenario_fixt"
    SETUP_VARIATION = "setup_variation_fixt"
    SETUP_TESTCASE = "setup_testcase_fixt"

    SCENARIO_SESSION = "scenario_session_fixt"
    SCENARIO_SETUP = "scenario_setup_fixt"
    SCENARIO_SCENARIO = "scenario_scenario_fixt"
    SCENARIO_VARIATION = "scenario_variation_fixt"
    SCENARIO_TESTCASE = "scenario_testcase_fixt"

