from __future__ import annotations

from _balder.device import Device


class VDevice(Device):
    """
    This is the main class for vDevices that will be used as inner classes in :class:`Feature` classes.
    """
