from .feature_i import SetupFeatureI
from .feature_ii import SetupFeatureII
from .feature_of_relevance_lvl3 import SetupFeatureOfRelevanceLvl3

__all__ = [
    "SetupFeatureI",
    "SetupFeatureII",
    "SetupFeatureOfRelevanceLvl3"
]
